#ifdef USE_PRAGMA_IDENT_SRC
#pragma ident "@(#)vm_version.cpp	1.55 07/05/05 17:07:03 JVM"
#endif
/*
 * Copyright 1998-2007 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 *  
 */

# include "incls/_precompiled.incl"
# include "incls/_vm_version.cpp.incl"

const char* Abstract_VM_Version::_s_vm_release = Abstract_VM_Version::vm_release();
const char* Abstract_VM_Version::_s_internal_vm_info_string = Abstract_VM_Version::internal_vm_info_string();
bool Abstract_VM_Version::_supports_cx8 = false;
unsigned int Abstract_VM_Version::_logical_processors_per_package = 1U;

#ifndef HOTSPOT_RELEASE_VERSION
  #error HOTSPOT_RELEASE_VERSION must be defined
#endif
#ifndef HOTSPOT_BUILD_VERSION
  #error HOTSPOT_BUILD_VERSION must be defined
#endif
#ifndef HOTSPOT_BUILD_TARGET
  #error HOTSPOT_BUILD_TARGET must be defined
#endif

#ifdef PRODUCT
  #define VM_RELEASE HOTSPOT_BUILD_VERSION
#else
  #define VM_RELEASE HOTSPOT_BUILD_VERSION "-" HOTSPOT_BUILD_TARGET
#endif

// HOTSPOT_BUILD_VERSION must follow the release version naming convention
// n.n.n[_uu][-<identifier>]-bnn
int Abstract_VM_Version::_vm_major_version = 0;
int Abstract_VM_Version::_vm_minor_version = 0;
int Abstract_VM_Version::_vm_micro_version = 0;
int Abstract_VM_Version::_vm_build_number = 0;
int Abstract_VM_Version::_vm_update_version = 0;
char Abstract_VM_Version::_special_update_version = '\0';
bool Abstract_VM_Version::_initialized = false;

void Abstract_VM_Version::initialize() {
  if (_initialized) {
    return;
  }

  const char* vm_version_string = HOTSPOT_BUILD_VERSION;
  const char* release_version_string = HOTSPOT_RELEASE_VERSION;
  
  const char* vm_version;
  if (strlen(vm_version_string) >= 5 &&
      isdigit(vm_version_string[0]) && 
      isdigit(vm_version_string[2]) &&
      isdigit(vm_version_string[4]) &&
      vm_version_string[1] == '.' && 
      vm_version_string[3] == '.') { 
    vm_version = vm_version_string;
  } else {
    // Internal builds e.g. PRT builds - use the current release version string
    vm_version = release_version_string;
  }

  _vm_major_version = vm_version[0] - '0'; 
  _vm_minor_version = vm_version[2] - '0'; 
  _vm_micro_version = vm_version[4] - '0'; 

  const char* p = &vm_version[5];
  while (*p != '\0') { 
    char c = *p;
    switch (c) {
      case '_': {
        // parse _uu
        if (strlen(p) >= 3 && isdigit(p[1]) && isdigit(p[2])) {
          char u[3];
          u[0] = p[1];
          u[1] = p[2];
          u[2] = '\0';
          _vm_update_version = atoi(u);
          p = p + 3;
          c = *p;
          if (c >= 'a' && c <= 'z') {
            _special_update_version = c;
            p++;
          }
        }
        break;
      }
      case '-': {
        // parse [-<identifier>]-bnn where bnn is the last on the version string
        if (strlen(p) == 4 && p[1] == 'b' && isdigit(p[2]) && isdigit(p[3])) {
          char b[3];
          b[0] = p[2];
          b[1] = p[3];
          b[2] = '\0';
          _vm_build_number = atoi(b);
          p = p + 4;
        } else {
          // skip -<identifier>
          c = *(++p);
          while (c != '-' && c != '\0') {
            c = *(++p);
          }
        } 
        break;
      }
      default: {
        // any other character - done parsing
        p = ""; 
      }
    }
  }
  _initialized = true;
}

#if defined(_LP64)
  #ifdef TIERED
  #define VMTYPE "64-Bit Tiered"
  #else
  #define VMTYPE COMPILER1_PRESENT("64-Bit Client")   \
                 COMPILER2_PRESENT("64-Bit Server")   
  #endif // TIERED
#else
  #ifdef TIERED
  #define VMTYPE "Tiered"
  #else
    #define VMTYPE COMPILER1_PRESENT("Client")       \
                   COMPILER2_PRESENT("Server")
  #endif // TIERED
#endif


#ifndef HOTSPOT_VM_DISTRO
  #error HOTSPOT_VM_DISTRO must be defined
#endif
#define VMNAME HOTSPOT_VM_DISTRO " " VMTYPE " VM"

const char* Abstract_VM_Version::vm_name() {
  return VMNAME;
}


const char* Abstract_VM_Version::vm_vendor() {
#ifdef VENDOR
  return XSTR(VENDOR);
#else
  return "Sun Microsystems Inc.";
#endif
}


const char* Abstract_VM_Version::vm_info_string() {
  switch (Arguments::mode()) {
    case Arguments::_int:
      return UseSharedSpaces ? "interpreted mode, sharing" : "interpreted mode";
    case Arguments::_mixed:
      return UseSharedSpaces ? "mixed mode, sharing"       :  "mixed mode";
    case Arguments::_comp:
      return UseSharedSpaces ? "compiled mode, sharing"    : "compiled mode";
  };
  ShouldNotReachHere();
  return "";
}

// NOTE: do *not* use stringStream. this function is called by 
//       fatal error handler. if the crash is in native thread,
//       stringStream cannot get resource allocated and will SEGV.
const char* Abstract_VM_Version::vm_release() {
  return VM_RELEASE;
}

#define OS       LINUX_ONLY("linux")             \
                 WINDOWS_ONLY("windows")         \
                 SOLARIS_ONLY("solaris")

#define CPU      IA32_ONLY("x86")                \
                 IA64_ONLY("ia64")               \
                 AMD64_ONLY("amd64")             \
                 SPARC_ONLY("sparc")

const char *Abstract_VM_Version::vm_platform_string() {
  return OS "-" CPU;
}

const char* Abstract_VM_Version::internal_vm_info_string() {
  #ifndef HOTSPOT_BUILD_USER
    #define HOTSPOT_BUILD_USER unknown
  #endif

  #ifndef HOTSPOT_BUILD_COMPILER
    #ifdef _MSC_VER
      #if   _MSC_VER == 1100
        #define HOTSPOT_BUILD_COMPILER "MS VC++ 5.0"
      #elif _MSC_VER == 1200
        #define HOTSPOT_BUILD_COMPILER "MS VC++ 6.0"
      #else
        #define HOTSPOT_BUILD_COMPILER "unknown MS VC++:" XSTR(_MSC_VER)
      #endif
    #elif defined(__SUNPRO_CC)
      #if   __SUNPRO_CC == 0x520
        #define HOTSPOT_BUILD_COMPILER "Workshop 5.2 compat=" \
          XSTR(__SUNPRO_CC_COMPAT)
      #elif __SUNPRO_CC == 0x500
        #define HOTSPOT_BUILD_COMPILER "Workshop 5.0 compat=" \
          XSTR(__SUNPRO_CC_COMPAT)
      #elif __SUNPRO_CC == 0x420
        #define HOTSPOT_BUILD_COMPILER "Workshop 4.2"
      #else
        #define HOTSPOT_BUILD_COMPILER "unknown Workshop:" XSTR(__SUNPRO_CC)
      #endif
    #elif defined(__GNUC__)
        #define HOTSPOT_BUILD_COMPILER "gcc " __VERSION__
    #else
      #define HOTSPOT_BUILD_COMPILER "unknown compiler"
    #endif
  #endif


  return VMNAME " (" VM_RELEASE ") for " OS "-" CPU
         ", built on " __DATE__ " " __TIME__ 
         " by " XSTR(HOTSPOT_BUILD_USER) " with " HOTSPOT_BUILD_COMPILER;
}

unsigned int Abstract_VM_Version::jvm_version() {
  return ((Abstract_VM_Version::vm_major_version() & 0xFF) << 24) |
         ((Abstract_VM_Version::vm_minor_version() & 0xFF) << 16) |
         ((Abstract_VM_Version::vm_micro_version() & 0xFF) << 8)  |
         (Abstract_VM_Version::vm_build_number() & 0xFF);
}


void VM_Version_init() {
  VM_Version::initialize();

#ifndef PRODUCT
  if (PrintMiscellaneous && Verbose) {
    os::print_cpu_info(tty);
  }
#endif
}
