/* 
 * Copyright (c) 2007 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 * 
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 * 
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */
/**
 * @(#)CheckDataVersion.java	1.2 07/04/06
 *
 * Check the consistency between the regression tests and the currency data in the JRE
 */

import java.io.*;
import java.lang.reflect.*;
import java.security.*;

class CheckDataVersion {
    static final String datafile = "tablea1.txt";
    static final String VERSIONKEY = "VERSION=";
    static String version;
    static boolean checked = false;

    static void check() {
        if (!checked) {
            try {
	        FileReader fr = new FileReader(new File(System.getProperty("test.src", "."), datafile));
	        BufferedReader in = new BufferedReader(fr);
	        String line;

	        while ((line = in.readLine()) != null) {
	            if (line.startsWith(VERSIONKEY)) {
		        version = line.substring(VERSIONKEY.length());
		        break;
		    }
	        }
	    } catch (IOException ioe) {
		throw new RuntimeException(ioe);
	    }

	    AccessController.doPrivileged(new PrivilegedAction<Object>() {
		public Object run() {
		    try {
			Class curdata = Class.forName("java.util.CurrencyData");
			Field f = curdata.getDeclaredField("version");
			f.setAccessible(true);
			String jreVersion = (String)f.get(null);
			if (!version.equals(jreVersion)) {
			    throw new RuntimeException("Test data and JRE's currency data are inconsistent.  test: "+version+" JRE: "+jreVersion);
			}
	    	    } catch (ClassNotFoundException cnfe) {
			throw new RuntimeException(cnfe);
	    	    } catch (NoSuchFieldException nsfe) {
			throw new RuntimeException(nsfe);
	    	    } catch (IllegalAccessException iae) {
			throw new RuntimeException(iae);
	    	    }    
		    return null;
		}
	    });
	    checked = true;
	}
    }
}
