/*
 * Copyright 2006 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

/**
 * @test 1.4 07/05/05
 * @bug 6436919
 * @summary check that XML Signatures can be generated and validated with 
 *  SecurityManager enabled and default policy
 * @author Sean Mullan
 */
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.util.Collections;
import javax.xml.crypto.dsig.*;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.dsig.spec.TransformParameterSpec;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XMLDSigWithSecMgr {

    public static void main(String[] args) throws Exception {

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	dbf.setNamespaceAware(true);
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document doc = db.newDocument();
        Element envelope = doc.createElementNS
            ("http://example.org/envelope", "Envelope");
        envelope.setAttributeNS("http://www.w3.org/2000/xmlns/", 
	    "xmlns", "http://example.org/envelope");
        doc.appendChild(envelope);

        KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA");
        KeyPair kp = kpg.genKeyPair();

        //XXX make sure we don't accidentally pick up user policy
        System.setSecurityManager(new SecurityManager());

        try {
            // generate a signature with SecurityManager enabled
            XMLSignatureFactory fac = XMLSignatureFactory.getInstance();
            Reference ref = fac.newReference
                ("", fac.newDigestMethod(DigestMethod.SHA1, null),
		 Collections.singletonList
		    (fac.newTransform(Transform.ENVELOPED, 
		     (TransformParameterSpec) null)), null, null);
            SignedInfo si = fac.newSignedInfo(
                fac.newCanonicalizationMethod
                    (CanonicalizationMethod.INCLUSIVE,
                     (C14NMethodParameterSpec) null),
                fac.newSignatureMethod(SignatureMethod.RSA_SHA1, null),
                Collections.singletonList(ref));
            XMLSignature sig = fac.newXMLSignature(si, null);
            DOMSignContext dsc = new DOMSignContext(kp.getPrivate(), envelope);
            sig.sign(dsc);

            // validate a signature with SecurityManager enabled
            DOMValidateContext dvc = new DOMValidateContext
		(kp.getPublic(), envelope.getFirstChild());
            sig = fac.unmarshalXMLSignature(dvc);
            if (!sig.validate(dvc)) {
		throw new Exception
		    ("XMLDSigWithSecMgr signature validation FAILED");
	    }
        } catch (SecurityException se) {
            throw new Exception("XMLDSigWithSecMgr FAILED", se);
        }
    }
}

